class TokenValidationError(Exception):
    pass
class NotFoundError(Exception):
    pass
class BadRequestError(Exception):
    pass
class TaskNotFoundError(Exception):
    pass
class UnauthorizedError(Exception):
    pass
class ListenerNotFoundError(Exception):
    pass
class ParameterNotFound(Exception):
    pass